import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

public class MainClass 
{


	public static class MeetLOG implements Comparable<MeetLOG>, Cloneable
	{
		private int id;
		private double out;
		private double in;
		private double beRejected;
		private double trust;
		private double trustIn;
		private double trustOut;
		
		private static final boolean IS_LOG_TRUST_INCREASE = true;
		private static final boolean IS_LINEAR_TRUST_INCREASE = false;
		private static final boolean IS_LOG_WANING = false;
		private static final boolean IS_LINEAR_WANING = true;
		private static final boolean IS_LOG_TRUST_DECREASE = true;
		private static final boolean IS_LINEAR_TRUST_DECREASE = false;
		
		//TODO
		public static double MAX_COMPRESS_RATIO = 10.0d;// weak ties preferred
		private static final double MIN_COMPRESS_RATIO = 0.0d;
		public static final double MAX_TRUST = 100;
		private static final double COMPRESSION_INTERVAL = 
			(MAX_COMPRESS_RATIO - MIN_COMPRESS_RATIO) / MAX_TRUST;
		private static final double MIN_WARNING = 8.0d;
		private static final double WARNING_C = 0.001d;
		

		private static final double RISK_RATE_CONS = 0.02d;
		
		private static final double TRUST_INCREASE_STEP_LINEARLY = 10;//40.0d for linear increase-log waning;
		private static final double TRUST_DECREASE_STEP_LINEARLY = 1.0d;
		//TODO
		public static double TRUST_WANINE_STEP_LINEARLY = 0.08d;//weak ties preferred
		
		MeetLOG(int id)
		{
			this.id = id;
			in = 0;
			out = 0;
			trust = 0.0d;
			trustIn = 0.0d;
			trustOut = 0.0d;
			beRejected = 0.0d;
		}
		
		boolean Equals(Object o)
		{
			if(o instanceof MeetLOG && ((MeetLOG)o).getID() == id)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
		
		public int getID()
		{
			return id;	
		}
		
		public void increaseTrust()
		{
			if(IS_LOG_TRUST_INCREASE)
			{
				trust = trust + Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust);
			}
			else if(IS_LINEAR_TRUST_INCREASE)
			{
				trust = trust + TRUST_INCREASE_STEP_LINEARLY;
			}	
			else
			{
				
			}
		}
		
		public void increaseTrustIn()
		{
			if(IS_LOG_TRUST_INCREASE)
			{
				trustIn = trustIn + Math.max(0, 
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn);
			}
			else if(IS_LINEAR_TRUST_INCREASE)
			{
				trustIn = trustIn + TRUST_INCREASE_STEP_LINEARLY;
			}
			else
			{
				
			}
		}

		public void increaseTrustOut()
		{
			if(IS_LOG_TRUST_INCREASE)
			{
				trustOut = trustOut + Math.max(0, 
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut);
			}
			else if(IS_LINEAR_TRUST_INCREASE)
			{
				trustOut = trustOut + TRUST_INCREASE_STEP_LINEARLY;
			}
			else
			{
				
			}
		}
		
		
		public void addIn()
		{
			in++;
		}

		public void addOut()
		{
			out++;
		}
		
		public void decreaseTrust()
		{
			if(IS_LOG_TRUST_DECREASE)
			{
				trust = Math.max(trust - Math.max(0, 
					(MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust)), 0);
			}
			else if(IS_LINEAR_TRUST_DECREASE)
			{
				trust = Math.max(0,trust - TRUST_DECREASE_STEP_LINEARLY);
			}
			else
			{
				
			}
		}
		public void decreaseTrustIn()
		{
			if(IS_LOG_TRUST_DECREASE)
			{
				trustIn = Math.max(trustIn - Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn) / 4, 0);
			}
			else if(IS_LINEAR_TRUST_DECREASE)
			{
				trustIn = Math.max(0,trustIn - TRUST_DECREASE_STEP_LINEARLY);
			}
			else
			{
				
			}

		}
		public void decreaseTrustOut()
		{
			if(IS_LOG_TRUST_DECREASE)
			{
				trustOut = Math.max(trustOut - Math.max(0,
					MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut), 0);
			}
			else if(IS_LINEAR_TRUST_DECREASE)
			{
				trustOut = Math.max(0,trustOut - TRUST_DECREASE_STEP_LINEARLY);
			}
			else
			{
				
			}
		}
		
		public double getInteraction()
		{
			return in + out;
		}
		
		public double getIn()
		{
			return in;
		}
		
		public void setIn(double in)
		{
			this.in = in;
		}
		
		public double getOut()
		{
			return out;
		}

		public void setOut(double out)
		{
			this.out = out;
		}
		
		public double getTrust()
		{
			return trust;
		}

		public double getRisk()
		{
			return out * RISK_RATE_CONS;
		}
		
		public double getDefenseProb()
		{
			return beRejected / (out + beRejected);
		}
		
		public void increaseBeRejected()
		{
			beRejected++;
		}
		
		public void setTrust(double trust)
		{
			this.trust = trust;
		}

		public void setTrustIn(double trustIn)
		{
			this.trustIn = trustIn;
		}

		public void setTrustOut(double trustOut)
		{
			this.trustOut = trustOut;
		}
		
		public void waning()
		{
			if(IS_LOG_WANING)
			{
				trust = Math.max(0, trust - Math.max(0,
						MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trust));
				trustIn = Math.max(0, trustIn - Math.max(0,
						MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustIn));
				trustOut = Math.max(0, trustOut - Math.max(0,
						MAX_COMPRESS_RATIO - COMPRESSION_INTERVAL * trustOut));
			}
			else if(IS_LINEAR_WANING)
			{
				trust = Math.max(0, trust - TRUST_WANINE_STEP_LINEARLY);
				trustIn = Math.max(0, trustIn - TRUST_WANINE_STEP_LINEARLY);
				trustOut = Math.max(0, trustOut - TRUST_WANINE_STEP_LINEARLY);
			}
			else
			{
				
			}		
		}
		
		 public int compareTo(MeetLOG o) 
		 throws ClassCastException 
		 {
			 double comparedTrust = o.getTrust();
			 if(trust > comparedTrust)
			 {
				 return 1;
			 }
			 else if(trust < comparedTrust)
			 {
				 return -1;
			 }
			 else
			 {
				 return 0;
			 }
		 }
	}

	public static class Ego implements Cloneable 
	{
		public double scorewellbeing;
		public double scoreResource;
		public double scoreAlliance;
		public double scoreRisk;
		public double scoreStress;
		private int id;
		private double eatCount;
		private double meetCount;
		private double fitnessWellBeing;
		private double fitnessResource; 
		private double fitnessRisk; 
		private double fitnessAlliance; 
		private double fitnessStress; 
		private List<MeetLOG> allMeetLog;
		private List<MeetLOG> kinsMeetLog;
		private int interIteration;
		private int strongTies;
		private int weakTies;
		private double trust;
		private double trustStrong;
		private double trustWeak;
		
		private Map<Integer, double[]> trustTrace;
		private boolean isTrustTrace;
		private int currentMetID;
		private boolean metSucceed;
		private String[] actionTrace;		
		
		//eat ratio
		private double eatRatio;		
		//eat ratio
		private double minDefectRate;
		
		//policies
		//1. strong ties preferred
		private boolean strongTiesPreferred;
		//2. weak ties preferred
		private boolean weakTiesPreferred;
		//3. stage preferred
		private boolean stagePreferred;
		//4. medium preferred
		private boolean mediumPreferred;
		//5. shuffle preferred
		private boolean shufflePreferred;		
		
		Ego(int id)
		{
			this.id = id;
			eatCount = 0;
			meetCount = 0;
			
			allMeetLog = new ArrayList<MeetLOG>();
			kinsMeetLog = new ArrayList<MeetLOG>();
			interIteration = 0;
			
			strongTiesPreferred = false;
			weakTiesPreferred = false;
			stagePreferred = false;
			mediumPreferred = false;
			shufflePreferred = false;	
			
			fitnessResource = 0;
			fitnessWellBeing = 0;
			fitnessRisk = 0;
			fitnessAlliance = 0; 
			fitnessStress = 0; 
			interIteration = 0;
		}	
		
		Ego(int id, Ego ego)
		{
			this.id = id;
			eatCount = 0;
			meetCount = 0;
			
			minDefectRate = ego.getDefectRate();
			eatRatio = ego.getEatMeetRatio();	
			
			allMeetLog = new ArrayList<MeetLOG>();
			kinsMeetLog = new ArrayList<MeetLOG>();		
			interIteration = 0;	
			
			strongTiesPreferred = ego.isStrongTiesPreferred();
			weakTiesPreferred = ego.isWeakTiesPreferred();
			stagePreferred = ego.isStagePreferred();
			mediumPreferred = ego.isMediumTiesPreferred();
			shufflePreferred = ego.isShufflePreferred();
			
			fitnessResource = 0;
			fitnessWellBeing = 0;
			fitnessRisk = 0;
			fitnessAlliance = 0; 
			fitnessStress = 0; 
			interIteration = 0;
		}

		public void setEatRatio(double eatRatio)
		{
			this.eatRatio = eatRatio;
		}
		

		public void setMinDefectRate(double minDefectRate)
		{
			this.minDefectRate = minDefectRate;
		}
		
		public void setTrustTrace()
		{
			isTrustTrace = true;
			trustTrace = new HashMap<Integer, double[]>();
			actionTrace = new String[Sociaty.iterations];
		}
		
		public void reset()
		{			
			eatCount = 0;
			meetCount = 0;
			fitnessResource = 0;
			fitnessWellBeing = 0;
			fitnessRisk = 0;
			fitnessAlliance = 0; 
			fitnessStress = 0; 
			fitnessAlliance = 0; 
			fitnessStress = 0; 
			interIteration = 0;
			strongTies = 0;
			weakTies = 0;
			trust = 0;
			fitnessRisk = 0;
			fitnessStress = 0;
			trustStrong = 0;
			trustWeak = 0;
			
			MeetLOG log = null;
			allMeetLog.clear();
			for(int i = 0; i < kinsMeetLog.size(); i++)
			{
				log = kinsMeetLog.get(i);
				if(log.getTrust() >= Sociaty.strongTiesThr)
				{
					double newTrust = Math.max(Sociaty.trustThrKins,log.getTrust());
					log.setTrust(newTrust);
					log.setTrustIn(newTrust);
					log.setTrustOut(newTrust);
					allMeetLog.add(log);
				}
				else
				{
					kinsMeetLog.remove(i);
					i--;
				}
			}
		}
		
		public List<MeetLOG> getAllMeetLog()
		{
			return allMeetLog;
		}
		
		public List<MeetLOG> getKinsMeetLog()
		{
			return kinsMeetLog;
		}
		
		public double getMeetCount()
		{
			return meetCount;
		}
		
		public double getEatCount()
		{
			return eatCount;
		}
		
		public int getInterIteration()
		{
			return interIteration;
		}

		public void setMediumTiesPreferred()
		{
			mediumPreferred = true;
		}
		public void setStrongTiesPreferred()
		{
			strongTiesPreferred = true;
		}
		public void setShufflePreferred()
		{
			shufflePreferred = true;
		}
		public void setWeakTiesPreferred()
		{
			weakTiesPreferred = true;
		}
		public void setStagePreferred()
		{
			stagePreferred = true;
		}
		public boolean isStrongTiesPreferred()
		{
			return strongTiesPreferred;
		}
		public boolean isWeakTiesPreferred()
		{
			return weakTiesPreferred;
		}
		public boolean isStagePreferred()
		{
			return stagePreferred;
		}
		public boolean isMediumTiesPreferred()
		{
			return mediumPreferred;
		}
		public boolean isShufflePreferred()
		{
			return shufflePreferred;
		}
		
		public void runATurn(List<Ego> egoes, int g, int i) throws Exception
		{
			double rand = Math.random();
			if(rand < eatRatio)
			{
				eat();
				metSucceed = false;
				if(isTrustTrace)
				{
					actionTrace[i]="eat";
				}			
			}
			else
			{
				meet(egoes, g, i);
				if(isTrustTrace)
				{
					actionTrace[i] = "meet "+currentMetID;
					if(metSucceed)
					{
						actionTrace[i] += " cooperated";
					}
					else
					{
						actionTrace[i] += " rejected";
					}
				}		
			}		
			//calculate fitness and update eat/meet ratio
			calculateFitness();		
			interIteration++;	
			
			//trace trust
			if(isTrustTrace)
			{
				double[] trusts = null;
				MeetLOG log = null;
				int logID;
				for(int j = 0; j < allMeetLog.size(); j++)
				{
					log = allMeetLog.get(j);
					logID = log.getID();				
					Iterator<Integer> it = trustTrace.keySet().iterator();
					Integer trustTraceID = null;
					boolean cont = true;
					while(cont && it.hasNext())
					{
						trustTraceID = it.next();
						if(trustTraceID.intValue() == logID)
						{
							trusts = trustTrace.get(trustTraceID);
							cont = false;
						}
					}
					if(cont)
					{
						trusts = new double[Sociaty.iterations];	
						trustTrace.put(new Integer(logID), trusts);
					}
					trusts[i] = log.getTrust();
				}
			}		
		}	
		
		public void debudPrintTrust() throws IOException
		{
			if(isTrustTrace)
			{
	//number of weak and medium ties to output 
	int numberWeak = 3;
	int numberMedium = 3;
	int weakCount = 0;
	int mediumCount = 0;

				Integer trustTraceID = null;
				double[] trust = null;
				Iterator<Integer> it = trustTrace.keySet().iterator();

				while(it.hasNext())
				{
					trustTraceID = it.next();
					trust = trustTrace.get(trustTraceID);
					
					if(trust[Sociaty.iterations - 1] > Sociaty.strongTiesThr)
					{
						printOneTrace(trustTraceID, trust);
					}
					
					else if(trust[Sociaty.iterations - 1] > Sociaty.mediumTiesThr )
					{
						if(mediumCount < numberMedium)
						{
							printOneTrace(trustTraceID, trust);
							mediumCount++;			
						}		
					}
					else
					{
						if(weakCount < numberWeak)
						{
							if(mediumCount < numberMedium)
							{
								printOneTrace(trustTraceID, trust);
								weakCount++;			
							}	
						}					
					}

				}
			}
		}
		
		private void printOneTrace(Integer trustTraceID, double[] trust) 
		throws IOException
		{
			String filename = id + "-" + trustTraceID.intValue() + ".txt";
			BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(filename));
			bufferedWriter.append("ego " + trustTraceID.intValue() + "\n");
			for(int i = 0; i < Sociaty.iterations; i++)
			{
				bufferedWriter.append(trust[i] + "\n");
			}
			bufferedWriter.close();
		}
		
		public double getStrongTrust()
		{
			double st = 0.0d;
			int c = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				double t = allMeetLog.get(i).getTrust();
				if(t >= Sociaty.strongTiesThr)
				{
					st += t;
					c++;
				}
			}
			
			if(c > 0)
			{
				return st / c;
			}
			else
			{
				return 0;
			}
		}
		
		public double getWeakTrustWithoutMediumTies()
		{
			return trustWeak;
		}
		
		public double getWeakTrustWithMediumTies()
		{
			double wt = 0.0d;
			int c = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				double t = allMeetLog.get(i).getTrust();
				if(t < Sociaty.mediumTiesThr)
				{
					wt += t;
					c++;
				}
			}
			if(c > 0)
			{
				return wt / c;
			}
			else
			{
				return 0;
			}
			
		}
		
		public double getMediumTrust()
		{
			double mt = 0.0d;
			int c = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				double t = allMeetLog.get(i).getTrust();
				if(t >= Sociaty.mediumTiesThr && t < Sociaty.strongTiesThr)
				{
					mt += t;
					c++;
				}
			}
			
			if(c > 0)
			{
				return mt / c;
			}
			else
			{
				return 0;
			}
			
		}
		
		public double getTrust()
		{
			return trust;
		}

		public void removeLog(int removedID)
		{
			boolean cont = true;
			for(int i = 0; cont && i < allMeetLog.size(); i++)
			{
				MeetLOG log = allMeetLog.get(i);
				if(log.getID() == removedID)
				{
					allMeetLog.remove(log);
					cont = false;
				}
			}		
			
			cont = true;
			for(int i = 0; cont && i < kinsMeetLog.size(); i++)
			{
				MeetLOG log = kinsMeetLog.get(i);
				if(log.getID() == removedID)
				{
					kinsMeetLog.remove(log);
					cont = false;
				}
			}
			calculateFitness();
		}
		
		public void calculateFitness()
		{
			clearZeroTrustTies();
			
			double totalIn = 0.0d;
			double totalOut = 0.0d;
			
			fitnessWellBeing = 0;
			strongTies = 0;
			trust = 0;
			fitnessRisk = 0;
			fitnessStress = 0;
			trustStrong = 0;
			trustWeak = 0;
			double averageTrust = 0;
			int ties = 0;
			if(allMeetLog.size() > 0)
			{
				for(int i = 0; i < allMeetLog.size(); i++)
				{ 
					if(allMeetLog.get(i).getTrust()> 0)
					{
						ties++;
					}
					
					if(allMeetLog.get(i).getTrust() >= Sociaty.strongTiesThr)
					{
						strongTies++;
						trustStrong += allMeetLog.get(i).getTrust();
					}	
					else
					{
						trustWeak += allMeetLog.get(i).getTrust();
					}
					trust += allMeetLog.get(i).getTrust();
					totalIn += allMeetLog.get(i).getIn();
					totalOut += allMeetLog.get(i).getOut();
					fitnessWellBeing += allMeetLog.get(i).getTrust();
				}
				averageTrust = fitnessWellBeing / ties;
				fitnessWellBeing = (fitnessWellBeing * totalIn / totalOut) / ties;
				if(Double.isInfinite(fitnessWellBeing))
				{
					fitnessWellBeing = 0;
				}
				
			}
			weakTies = ties - strongTies;
			fitnessResource = (1 + allMeetLog.size() / 3) * eatCount / 100.0;
			fitnessRisk = averageTrust + 0.005 * meetCount;
			fitnessAlliance = averageTrust + 0.05 * ties;
			fitnessStress = 0.1 * ties;
			clearZeroTrustTies();
			
		}
		
		private void clearZeroTrustTies()
		{
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				if(allMeetLog.get(i).getTrust() <= 0)
				{
					allMeetLog.remove(i);
					i--;
				}
			}
		}

		public int getStrongTies()
		{
			int st = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				if(allMeetLog.get(i).getTrust() > Sociaty.strongTiesThr)
				{
					st++;
				}
			}
			return st;
		}
		
		public int getMediumTies()
		{
			int mt = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				if(allMeetLog.get(i).getTrust() > Sociaty.mediumTiesThr)
				{
					mt++;
				}
			}
			return mt - strongTies;
		}
		
	//version 1 without medium ties	
		public int getWeakTiesWithoutMudium()
		{
			return weakTies;
		}
	//version 2 with medium ties	
		public int getWeakTiesWithMudium()
		{
			int wt = 0;
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				if(allMeetLog.get(i).getTrust() <= Sociaty.mediumTiesThr)
				{
					wt++;
				}
			}
			return wt;
		}
		private void eat()
		{
			eatCount++;
		}
		
		public void addKins(Ego ego)
		{
			MeetLOG meetLog = new MeetLOG(ego.getID());
			meetLog.setTrust(Sociaty.trustThrKins);
			meetLog.setTrustIn(Sociaty.trustThrKins);
			meetLog.setTrustOut(Sociaty.trustThrKins);
			if(!hasKin(ego))
			{
				kinsMeetLog.add(meetLog);
			}
		}
		
		public int getNumberKins()
		{
			return kinsMeetLog.size();
		}
		
		private void meetStrong(List<Ego> egoes)
		{				
			double totalTrust = 0.0d;
			MeetLOG[] tmpLog = new MeetLOG[allMeetLog.size()];			
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				totalTrust += allMeetLog.get(i).getTrust();
				tmpLog[i] =  allMeetLog.get(i);
			}
			Arrays.sort(tmpLog);
			int selectedID = -1;
			double tmpTrust = 0.0d;
			double rand = Math.random() * totalTrust;
			boolean cont = true;
			for(int i = allMeetLog.size() - 1; cont && i >= 0; i--)
			{	
				tmpTrust += tmpLog[i].getTrust();
				if(rand <= tmpTrust)
				{
					selectedID = tmpLog[i].getID();
					cont = false;
				}
			}		
			tryToMeet(egoes, selectedID);
			
		}
		private void meetMedium(List<Ego> egoes)
		{		
			//meet medium
			double totalTrust = 0.0d;
			MeetLOG[] tmpLog = new MeetLOG[allMeetLog.size()];			
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				totalTrust += allMeetLog.get(i).getTrust();
				tmpLog[i] =  allMeetLog.get(i);
			}
			
			double halfTrust = totalTrust / 2;
			boolean cont = true;			
			double tmpTrust = 0.0d;		
			double preHalfTrust = 0.0d;
			int middlePos = -1;			
			
			Arrays.sort(tmpLog);				
			//find half trust position as medium, 
			//which is the beginning of the lower part (strong preferred)
			//above is weak preferred
			for(int i = 0; cont && i < tmpLog.length; i++)
			{
				preHalfTrust += tmpLog[i].getTrust();
				if(preHalfTrust >= halfTrust)
				{
					middlePos = i;
					preHalfTrust -= tmpLog[i].getTrust();
					cont = false;						
				}
			}
			
			
			int selectedID = -1;
			double rand = Math.random() * totalTrust;
			
			if(rand <= preHalfTrust)
			{				
				tmpTrust = 0.0d;
				cont = true;
				//for half strong preferred;
				for(int i = 0; cont && i < middlePos; i++)
				{
					tmpTrust += tmpLog[i].getTrust();
					if(rand <= tmpTrust)
					{
						selectedID = tmpLog[i].getID();
						cont = false;
					}
				}					
			}
			else
			{
				tmpTrust = preHalfTrust;
				cont = true;
				//for half weak preferred;
				for(int i = middlePos; cont && i < tmpLog.length; i++)
				{
					tmpTrust += tmpLog[i].getTrust();
					if(rand <= tmpTrust)
					{
						selectedID = tmpLog[allMeetLog.size() - 1 - i].getID();
						cont = false;
					}
				}
			}		
			
			tryToMeet(egoes, selectedID);
			

		}

		private void meetWeak(List<Ego> egoes)
		{		
			double totalTrust = 0.0d;
			MeetLOG[] tmpLog = new MeetLOG[allMeetLog.size()];			
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				totalTrust += allMeetLog.get(i).getTrust();
				tmpLog[i] =  allMeetLog.get(i);
			}
			Arrays.sort(tmpLog);
			int selectedID = -1;
			double tmpTrust = 0.0d;
			double rand = Math.random() * totalTrust;
			boolean cont = true;
			for(int i = allMeetLog.size() - 1; cont && i >= 0; i--)
			{	
				tmpTrust += tmpLog[i].getTrust();
				if(rand <= tmpTrust)
				{
					selectedID = tmpLog[allMeetLog.size() - 1 - i].getID();
					cont = false;
				}
			}		
			
			tryToMeet(egoes, selectedID);		

		}
		
		private void meetNew(List<Ego> egoes)
		{		
			Ego SelectedEgo = null;
			double rand = Math.random() * (Sociaty.population - allMeetLog.size() - 1);
			int count = 0;
			boolean cont = true;
			for(int i = 0; cont && i < egoes.size(); i++)
			{
				if(egoes.get(i).getID() != id)
				{
					boolean interCont = true;
					for(int j = 0; interCont && j < allMeetLog.size(); j++)
					{
						if(egoes.get(i).getID() == allMeetLog.get(j).getID())
						{
							interCont = false;
						}
					}
					if(interCont)
					{
						count++;
						if(rand < count)
						{
							SelectedEgo = egoes.get(i);
							cont = false;
						}
					}
				}
			}
			currentMetID = SelectedEgo.getID();
			if(SelectedEgo.cooperate(this))
			{
				addMeetLogOut(SelectedEgo);
				metSucceed = true;
			}
			else
			{
				rejectMeetLog(SelectedEgo);
				metSucceed = false;
			}
		}

		private void meetStage(List<Ego> egoes) throws Exception
		{		
			double totalLeftSteps = Sociaty.iterations - interIteration;
			MeetLOG[] tmpLog = new MeetLOG[allMeetLog.size()];
			double totalTrust = 0.0d;
			for(int i = 0; i < tmpLog.length; i++)
			{
				tmpLog[i] = allMeetLog.get(i);
				totalTrust += tmpLog[i].getTrust();
			}
			Arrays.sort(tmpLog);
			
			double rand = Math.random() * totalTrust;
			double chance = 0.0d;
			boolean cont = true;
			for(int i = 0; cont && i < tmpLog.length; i++)
			{
				chance += tmpLog[i].getTrust() - (tmpLog[i].getTrust() - 
						tmpLog[tmpLog.length - i - 1].getTrust()) / totalLeftSteps;
				if(rand < chance)
				{
					int selctedID = tmpLog[i].getID();
					Ego SelectedEgo = null;
					boolean interCont = true;
					for(int j = 0; interCont && j < egoes.size(); j++)
					{
						if(egoes.get(j).getID() == selctedID)
						{
							SelectedEgo = egoes.get(j);
							interCont = false;
						}
					}

					currentMetID = SelectedEgo.getID();
					if(SelectedEgo.cooperate(this))
					{
						addMeetLogOut(SelectedEgo);
						metSucceed = true;
					}
					else
					{
						rejectMeetLog(SelectedEgo);
						metSucceed = false;
					}
					cont = false;
				}
			}		
			if(cont)
			{
				throw new Exception();
			}
		}
		
		private void meetCompletelyRandomly(List<Ego> egoes)
		{		
			int selectedIndex = (int)Math.floor(Math.random() * egoes.size());
			while(egoes.get(selectedIndex).getID() == id)
			{
				selectedIndex = (int)Math.floor(Math.random() * egoes.size());
			}
			Ego SelectedEgo = egoes.get(selectedIndex);
			currentMetID = SelectedEgo.getID();
			if(SelectedEgo.cooperate(this))
			{
				addMeetLogOut(SelectedEgo);
				metSucceed = true;
			}
			else
			{
				rejectMeetLog(SelectedEgo);
				metSucceed = false;
			}
		}
		
		private void meet(List<Ego> egoes, int generation, int iteration) throws Exception
		{
			if(allMeetLog.size() == 0)
			{
				//seeding
				meetCompletelyRandomly(egoes);
			}	
			else
			{
				double rand0 = Math.random();
				if((rand0 > allMeetLog.size() / (double)Sociaty.population) 
						&& (allMeetLog.size() < Sociaty.population - 1))
				{
					meetNew(egoes);
				}
				else
				{				
					if(strongTiesPreferred)
					{
						meetStrong(egoes);
					}
					else if(weakTiesPreferred)
					{
						meetWeak(egoes);
					}
					else if(stagePreferred)
					{
						meetStage(egoes);
					}
					else if(mediumPreferred)
					{
						meetMedium(egoes);
					}
					else if(shufflePreferred)
					{
						int x = iteration % 2;
						if(x == 0)
						{
							meetStrong(egoes);
						}
						else if(x == 2)
						{
							meetMedium(egoes);
						}
//						else if(x == 3)
//						{
//							meetWeak(egoes);
//						}
						else
						{
							//nothing is done;
						}
					}
					else
					{
						double totalSteps = (Sociaty.generations - generation) 
								* Sociaty.iterations - iteration + interIteration;
						if(Math.random() <= (double)interIteration / totalSteps)
						{
							meetExisting(egoes);
						}
						else
						{
							meetCompletelyRandomly(egoes);
						}
					}
				}

			}
		}
		
		private void meetExisting(List<Ego> egoes)
		{	
			MeetLOG[] tmpMeetLog = new MeetLOG[allMeetLog.size()];
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				tmpMeetLog[i] = allMeetLog.get(i);
			}
			Arrays.sort(tmpMeetLog);

			double totalTrust = 0.0d;
			if( Math.random() < 0.25)
			{
				for(int i = 0; i < allMeetLog.size(); i++)
				{
					totalTrust += tmpMeetLog[i].getTrust();
				}
			}
			else
			{
				for(int i = 0; i < (double)allMeetLog.size() / 2; i++)
				{
					totalTrust += tmpMeetLog[i].getTrust();
				}
			}
			
			double tmpTrust = 0.0d;
			double rand = Math.random() * totalTrust;
			int i;
			for(i = 0; rand < tmpTrust && i < allMeetLog.size(); i++)
			{
				tmpTrust += tmpMeetLog[i].getTrust();
			}			
			tryToMeet(egoes, tmpMeetLog[i].getID());			

		}
		

		private void tryToMeet(List<Ego> egoes, int selectedID)
		{
			Ego SelectedEgo = null;
			boolean cont = true;
			for(int i = 0; cont && i < egoes.size(); i++)
			{
				if(egoes.get(i).getID() == selectedID)
				{
					SelectedEgo = egoes.get(i);
					cont = false;
				}
			}	
			currentMetID = SelectedEgo.getID();
			if(SelectedEgo.cooperate(this))
			{
				addMeetLogOut(SelectedEgo);	
				metSucceed = true;
			}
			else
			{
				rejectMeetLog(SelectedEgo);
				metSucceed = false; 
			}	
		}	
			
		private boolean cooperate(Ego focal)
		{
			
			if(Sociaty.DEFENSE)
			{
				//calculate defense probability defence policy and penalty
				double denfenseProability = 0.0d;
				int focalID = focal.getID();
				boolean cont = true;
				for(int i = 0; cont && i< allMeetLog.size(); i++)
				{
					if(allMeetLog.get(i).getID() == focalID)
					{
						denfenseProability = Math.max(minDefectRate,
								allMeetLog.get(i).getDefenseProb());
						cont = false;
					}
				}
				double r = Math.random();
				if(r < denfenseProability)
				{
					if(Sociaty.symmetrical)
					{
						rejectMeetLog(focal);
						if(Sociaty.rewardDefection)
						{
							eatCount++;
						}
					}
					return false;
				}
				else
				{
					meetCount++;
					addMeetLogIn(focal);
					return true;
				}			
			}
			else
			{
				meetCount++;
				addMeetLogIn(focal);
				return true;
			}
		}
		
		private void addMeetLogIn(Ego ego)
		{
			boolean added = false;
			for(int i = 0; !added && i < allMeetLog.size(); i++)
			{
				if(ego.getID() == allMeetLog.get(i).getID())
				{
					allMeetLog.get(i).increaseTrust();
					allMeetLog.get(i).increaseTrustIn();
					allMeetLog.get(i).addIn();
					added = true;
				}
			}
			if(!added)
			{
				//create a new log to add
				MeetLOG log = new MeetLOG(ego.getID());  
				log.increaseTrust();
				log.increaseTrustIn();
				log.addIn();
				allMeetLog.add(log);
			}
		}
		
		private void addMeetLogOut(Ego ego)
		{
			boolean added = false;
			for(int i = 0; !added && i < allMeetLog.size(); i++)
			{
				if(ego.getID() == allMeetLog.get(i).getID())
				{
					allMeetLog.get(i).increaseTrust();
					allMeetLog.get(i).increaseTrustOut();
					allMeetLog.get(i).addOut();
					added = true;
				}
			}
			if(!added)
			{
				//create a new log to add
				MeetLOG log = new MeetLOG(ego.getID());  
				log.increaseTrust();
				log.increaseTrustOut();
				log.addOut();
				allMeetLog.add(log);
			}
		}
		
		private void rejectMeetLog(Ego ego)
		{
			boolean rejected = false;
			for(int i = 0; !rejected && i < allMeetLog.size(); i++)
			{
				if(ego.getID() == allMeetLog.get(i).getID())
				{
					allMeetLog.get(i).decreaseTrust();
					allMeetLog.get(i).decreaseTrustOut();
					allMeetLog.get(i).increaseBeRejected();
					rejected = true;
				}
			}
			if(!rejected)
			{
				//create a new log to add
				MeetLOG log = new MeetLOG(ego.getID());  
				log.decreaseTrust();
				log.decreaseTrustOut();
				log.increaseBeRejected();
				allMeetLog.add(log);
			}
		}

		public void mutate()
		{
			//mutation on eat/meet ratio
			double r = Math.random();
			if(r < 0.3333d)
			{
				int slot = (int)Math.floor(Math.random() * Sociaty.mutationSlots);
				eatRatio  = Sociaty.minEMR + slot * Sociaty.gradEMR;
			}
			
			//mutation on defect rate
			else if(r < 0.6666d)
			{
				int slot = (int)Math.floor(Math.random() * Sociaty.mutationSlots);
				minDefectRate  = Sociaty.minD + slot * Sociaty.gradD;
			}
			
			//mutation on strategy
			else
			{
				if(stagePreferred)
				{
					stagePreferred = false;
					double rand = Math.random();
					if(rand < 0.3333)
					{					
						strongTiesPreferred = true;
					}
					else if(rand < 0.6666)
					{
						weakTiesPreferred = true;
					}
					else
					{
						mediumPreferred = true;
					}
				}
				else if(strongTiesPreferred)
				{
					strongTiesPreferred = false;
					double rand = Math.random();
					if(rand < 0.3333)
					{					
						stagePreferred = true;
					}
					else if(rand < 0.6666)
					{
						weakTiesPreferred = true;
					}
					else
					{
						mediumPreferred = true;
					}				
				}
				else if(weakTiesPreferred)
				{
					weakTiesPreferred = false;
					double rand = Math.random();
					if(rand < 0.3333)
					{					
						stagePreferred = true;
					}
					else if(rand < 0.6666)
					{
						strongTiesPreferred = true;
					}
					else
					{
						mediumPreferred = true;
					}				
				}
				else if(mediumPreferred)
				{
					mediumPreferred = false;
					double rand = Math.random();
					if(rand < 0.3333)
					{					
						stagePreferred = true;
					}
					else if(rand < 0.6666)
					{
						weakTiesPreferred = true;
					}
					else
					{
						strongTiesPreferred = true;
					}				
				}
			}
			
		}
		
		public void tiesDecay()
		{
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				allMeetLog.get(i).waning();
			}
			clearZeroTrustTies();
		}

		public boolean hasKin(Ego ego)
		{
			int egoID = ego.getID();
			for(int i = 0; i < kinsMeetLog.size(); i++)
			{
				if(egoID == kinsMeetLog.get(i).getID())
				{
					return true;
				}
			}
			return false;
		}

		public boolean hasMet(Ego ego)
		{
			int egoID = ego.getID();
			for(int i = 0; i < allMeetLog.size(); i++)
			{
				if(egoID == allMeetLog.get(i).getID())
				{
					return true;
				}
			}
			return false;
		}
		
		public int getID()
		{
			return id;
		}

		public double getEatMeetRatio()
		{
			return eatRatio;
		}
		
		public double getDefectRate()
		{
			return minDefectRate;
		}
		
		public double getResource()
		{
			return fitnessResource;
		}

		public double getWellbeing()
		{
			return fitnessWellBeing;
		}

		public double getAlliance()
		{
			return fitnessAlliance;
		}	

		public double getRisk()
		{
			return fitnessRisk;
		}
		
		public double getStress()
		{
			return fitnessStress;
		}
		
		public double getResourceScore()
		{
			return scoreResource;
		}

		public double getWellbeingScore()
		{
			return scorewellbeing;
		}

		public double getAllianceScore()
		{
			return scoreAlliance;
		}	

		public double getRiskScore()
		{
			return scoreRisk;
		}
		
		public double getStressScore()
		{
			return scoreStress;
		}
		

		public double getScore()
		{
			return Sociaty.rescouseWeight * scoreResource
				+ Sociaty.wellBeingWeight * scorewellbeing
				+ Sociaty.allianceWeight * scoreAlliance
				+ Sociaty.riskWeight * scoreRisk
				+ Sociaty.stressWeight * scoreStress;
		}
		
		
		public double getAllTies()
		{
			return weakTies + strongTies;
		}
		
		public String getStrategy()
		{
			if(stagePreferred)
			{
				return "stage";
			}
			else if(strongTiesPreferred)
			{
				return "strong";
			}
			else if(weakTiesPreferred)
			{
				return "weak";
			}
			else if(mediumPreferred)
			{
				return "medium";
			}
			else
			{
				return null;
			}
		}
		
		public void setStrategy(String strategy)
		{
			stagePreferred = false;
			strongTiesPreferred = false;
			weakTiesPreferred = false;
			mediumPreferred = false;
			if(strategy.equals("stage"))
			{
				stagePreferred = true;
			}
			else if(strategy.equals("strong"))
			{
				strongTiesPreferred = true;
			}
			else if(strategy.equals("weak"))
			{
				weakTiesPreferred = true;
			}
			else if(strategy.equals("medium"))
			{
				mediumPreferred = true;
			}
			else
			{
				;
			}
		}
	}

	public static class Sociaty 
	{
		private List<Ego> egoes;
		boolean printLCurve = true;
		boolean prrintIndividualTrace = true;
		boolean printAverage = true;
		
		public boolean SBH;
		
		public static final int population = 300;
		public static final int iterations = 3000;
		public static final int generations = 1;	
		
		public static boolean crossoverEM = false;
		public static boolean crossoverST = false;
		public static boolean crossoverDe = false;	
		

		public static final boolean DEFENSE = true;
		public static final boolean symmetrical = false;
		public static final boolean rewardDefection = true;
		
		
		public static final double averageNumberKins = 0;
		public static final double trustThrKins = 80.0d;

		private static final double crossoverRate = 0.9d; 
		private static final double mutationRate = 0.01d; 
		
		public static double[] eatMeetGenesIni = {0.125, 0.2, 0.25, 0.5};
		public static double[] DefectGenesIni = {0.01, 0.025, 0.05, 0.075};
		
		public static double maxEMR = 0.5d;
		public static double minEMR = 0.125d;

		public static double maxD = 0.075d;
		public static double minD = 0.01d;
		
		public static int mutationSlots = 1000;
		public static double gradEMR = (maxEMR - minEMR)/mutationSlots;
		public static double gradD = (maxD - minD)/mutationSlots;
		
			
		int[] g1;
		int[] g2;
		int[] g3;
		int[] g4;
		int[] g5;
		
		public static final double killRate = 0.2d;
		public static double strongTiesThr = 66;
		public static double mediumTiesThr = 33;
		
		public static double rescouseWeight;
		public static double wellBeingWeight;	
		public static double riskWeight;
		public static double allianceWeight;	
		public static double stressWeight;
		
		public Sociaty() 
		{
			g1 = new int[generations + 1];
			g2 = new int[generations + 1];
			g3 = new int[generations + 1];
			g4 = new int[generations + 1];
			g5 = new int[generations + 1];
			g1[0] = 150;//Stage
			g2[0] = 150;//Strong
			g3[0] = 0;//Weak
			g4[0] = 0;//Medium
			g5[0] = 0;//Shuffle
			egoes = new ArrayList<Ego>();
			Ego ego = null;		
			int bias = 0;
			for(int i = 0; i < g1[0]; i++)
			{
				ego = new Ego(i + bias);
				ego.setStagePreferred();
				egoes.add(ego);
			}		
			bias += g1[0];
			for(int i = 0; i < g2[0]; i++)
			{
				ego = new Ego(i + bias);
				ego.setStrongTiesPreferred();
				egoes.add(ego);
			}
			bias += g2[0];
			for(int i = 0; i < g3[0]; i++)
			{
				ego = new Ego(i + bias);
				ego.setWeakTiesPreferred();
				egoes.add(ego);
			}
			bias += g3[0];
			for(int i = 0; i < g4[0]; i++)
			{
				ego = new Ego(i + bias);
				ego.setMediumTiesPreferred();
				egoes.add(ego);
			}
			bias += g4[0];
			for(int i = 0; i < g5[0]; i++)
			{
				ego = new Ego(i + bias);
				ego.setShufflePreferred();
				egoes.add(ego);
			}		
		
			setEatRatio();
			setMinDefectRate();
		}

		
		public void setEatRatio()
		{
			for(int i = 0; i < population; i++)
			{
				int index = (int)Math.floor(Math.random() * eatMeetGenesIni.length);
				egoes.get(i).setEatRatio(eatMeetGenesIni[index]);
			}
		}
		
		public void setMinDefectRate()
		{
			for(int i = 0; i < population; i++)
			{
				int index = (int)Math.floor(Math.random() * DefectGenesIni.length);
				egoes.get(i).setMinDefectRate(DefectGenesIni[index]);
			} 
		}

		private void remove(Ego ego)
		{
			int removedID = ego.getID();
			for(int i = 0; i < egoes.size(); i++)
			{
				egoes.get(i).removeLog(removedID);
			}
			egoes.remove(ego);
		}
		public void reset()
		{
			//add kins
			if(averageNumberKins > 0)
			{
				for(int i = 0; i < population; i++)
				{
					if(egoes.get(i).getNumberKins() < averageNumberKins)
					{
						boolean cont = true;
						for(int j = i + 1; cont && j < population; j++)
						{
							if(egoes.get(j).getNumberKins() < averageNumberKins)
							{
								egoes.get(i).addKins(egoes.get(j));
								egoes.get(j).addKins(egoes.get(i));
								if(egoes.get(i).getNumberKins() < averageNumberKins)		
								{
									cont = true;
								}
								else
								{
									cont = false;
								}		
							}
						}
					}
				}
			}

			
			//reset and mutate
			for(int i = 0; i < population; i++)
			{
				egoes.get(i).reset();
				if(Math.random() < mutationRate)
				{
					egoes.get(i).mutate();
				}
			}
			
			//crossover
			Ego ego1 = null;
			Ego ego2 = null;
			for(int i = 0; i < population; i++)
			{
				if(Math.random() < crossoverRate)
				{
					if(ego1 == null)
					{
						ego1 = egoes.get(i);
					}
					else if(ego2 == null)
					{
						ego2 = egoes.get(i);
						crossover(ego1, ego2);
						ego1 = null;
						ego2 = null;					
					}
				}
			}
		}	

		public void crossover(Ego ego1, Ego ego2) 
		{		
			int div = 0;
			if(crossoverDe)
			{
				div++;
			}
			if(crossoverEM)
			{
				div++;
			}
			if(crossoverST)
			{
				div++;
			}
			double x = 1.0d / div;				
			double y = x;
			double rand = Math.random();
			boolean done = false;
			if(!done && crossoverST)
			{
				if(rand < y)
				{
					//crossover on strategy
					String strategy = ego1.getStrategy();
					ego1.setStrategy(ego2.getStrategy());
					ego2.setStrategy(strategy);
					done = true;
				}
				else
				{
					y += x;
				}			
			}
			if(!done && crossoverEM)
			{
				if(rand < y)
				{
					//crossover on eat meet ratio
					double rate = ego1.getEatMeetRatio();
					ego1.setEatRatio(ego2.getEatMeetRatio());
					ego2.setEatRatio(rate);
					done = true;
				}
				else
				{
					y += x;
				}	
			}

			if(!done && crossoverDe)
			{
				//crossover on defection rate
				double rate = ego1.getDefectRate();
				ego1.setMinDefectRate(ego2.getDefectRate());
				ego2.setMinDefectRate(rate);			
			}
		}
		
		public void run() 
		throws Exception
		{
			if(prrintIndividualTrace)
			{
				egoes.get(50).setTrustTrace();
				egoes.get(100).setTrustTrace();
				egoes.get(150).setTrustTrace();
				egoes.get(200).setTrustTrace();
				egoes.get(250).setTrustTrace();
			}
			
			
			int total = iterations * generations;
			double[] trustStrong = new double[total];
			double[] trustWeak = new double[total];
			double[] trustMedium = new double[total];
			double[] strongTies = new double[total];
			double[] weakTies = new double[total];
			double[] mediumTies = new double[total];
			
			double[] averageEatMeetRatio = new double[generations + 1];
			averageEatMeetRatio[0] = 0.26875;
			double[] averageDefectRatio = new double[generations + 1];
			averageDefectRatio[0] = 0.0475;
			
			double[][] trustFreq = new double[generations][100];
			
			
			for(int i = 0; i < generations; i++)
			{
				for(int j = 0; j < 100; j++)
				{
					trustFreq[i][j] = 0;
				}
			}
			
			Ego selectedEgo = null;
			
			String s = "index," +
			"average trust for strong ties," +
			"average trust for medium ties," +
			"average trust for weak ties," +
			"average strong ties, " +
			"average medium ties, " +
			"average weak ties," +
			"average total ties";
			System.out.println(s);
			
			
			for(int k = 0; k < generations; k++)
			{
				reset();

				for(int i = 0; i < iterations; i++)
				{
					for(int j = 0; j < population; j++)
					{
						selectedEgo = egoes.get(j);
						selectedEgo.tiesDecay();
					}	
					
					int ik = k * iterations + i;
					trustStrong[ik] = 0;
					trustWeak[ik] = 0;
					trustMedium[ik] = 0;
					strongTies[ik] = 0;
					weakTies[ik] = 0;
					mediumTies[ik] = 0;
					
					
					for(int j = 0; j < population; j++)
					{
						selectedEgo = egoes.get(j);
						selectedEgo.runATurn(egoes, k, i);

						trustStrong[ik] += selectedEgo.getStrongTrust();
						trustWeak[ik] += selectedEgo.getWeakTrustWithMediumTies();
						trustMedium[ik] += selectedEgo.getMediumTrust();
						strongTies[ik] += selectedEgo.getStrongTies();
						weakTies[ik] += selectedEgo.getWeakTiesWithMudium();
						mediumTies[ik] += selectedEgo.getMediumTies();
					}			
					
					trustStrong[ik] /= population;
					trustWeak[ik] /= population;
					trustMedium[ik] /= population;
					strongTies[ik] /= population;
					weakTies[ik] /= population;
					mediumTies[ik] /= population;
					if(printAverage)

					{
						System.out.print(ik + ",");
						System.out.print(trustStrong[ik] + ",");
						System.out.print(trustMedium[ik] + ",");
						System.out.print(trustWeak[ik] + ",");
						System.out.print(strongTies[ik] + ",");
						System.out.print(mediumTies[ik] + ",");
						System.out.print(weakTies[ik] + ",");
						System.out.println((strongTies[ik]+mediumTies[ik]+weakTies[ik]));
					}		
				}
				
				//for each generation; investigate the L shape relationship;
				//max trust = 100;
				if(printLCurve)
				{
					for(int i = 0; i < egoes.size(); i++)
					{
						List<MeetLOG> tmpLog = egoes.get(i).getAllMeetLog();
						for(int j = 0; j < tmpLog.size(); j++)
						{
							double tmpTrust = tmpLog.get(j).getTrust();
							if(tmpTrust > 0)
							{
								boolean cont = true;
								for(int ii = 0; cont && ii < 100; ii++)
								{
									if(tmpTrust <= (ii + 1))
									{
										trustFreq[k][ii]++;
										cont = false;
									}
								}
							}
							else
							{
								throw new Exception();
							}
						}		
					}
					
					for(int ii = 0; ii < 100; ii++)
					{
						trustFreq[k][ii] /= (double)egoes.size();
					}					
					for(int ii = 98; ii >= 0; ii--)
					{
						trustFreq[k][ii] += trustFreq[k][ii + 1];	
					}
					System.out.println();
					System.out.println();
					System.out.println();
					System.out.println("frequency,trust");			
					for(int ii = 0; ii < 100; ii++)
					{
						System.out.print(trustFreq[k][ii] + ",");
						System.out.println((ii + 1));
					}
					System.out.println();
				}
			}

			if(prrintIndividualTrace)
			{
				egoes.get(50).debudPrintTrust();
				egoes.get(100).debudPrintTrust();
				egoes.get(150).debudPrintTrust();
				egoes.get(200).debudPrintTrust();
				egoes.get(250).debudPrintTrust();
				System.out.println();
				System.out.println();
				System.out.println();
				System.out.println("Check log files for individual ego!");		
			}		
		}	
	}

	private static boolean run(Sociaty sociaty, 		
			double rescouseWeight, double wellBeingWeight, double allianceWeight, double riskWeight, double stressWeight,
			boolean crossoverST, boolean crossoverEM, boolean crossoverDe) 
	throws Exception
	{
		Sociaty.rescouseWeight = rescouseWeight;
		Sociaty.wellBeingWeight = wellBeingWeight;
		Sociaty.riskWeight = riskWeight;
		Sociaty.allianceWeight = allianceWeight;
		Sociaty.stressWeight = stressWeight;
		
		Sociaty.crossoverST = crossoverST;
		Sociaty.crossoverEM = crossoverEM;
		Sociaty.crossoverDe = crossoverDe;
		sociaty.run();
		System.out.println();
		System.out.println();
		return sociaty.SBH;
	}
	public static void main(String[] args) 
	throws Exception 
	{
		Sociaty sociaty = new Sociaty();
		run(sociaty, 0,0,0,0,0, true, true, true);	

	}

}
